classdef ActiveLearner
    %ActiveLearner Abstract class to represent active learning algorithms.
    %   This class is an interface for different query strategies. The
    %   class must have a constructor which waits for only one parameter a
    %   cellarray, which follows the format given back by the getParameters
    %   static method.
    % author: szkabel
    
    properties
        sizeOfInitialSet
    end
    
    methods (Abstract)        
        %proposeNextCell the most important function of ActiveLearners. If
        %it is called, it must propose samples for labeling to the oracle.
        %The function should not have to jump the GUI only give back the
        %possible samples by providing its identification (plate, image,
        %and cell number). They're all by numbers. It can happen that we
        %want more than one cell to propose, we can ask for that by setting
        %nofCells.
        %   INPUTS:       
        %   nofCells: an integer the required amount of proposed cells.
        %   (warning: it is not provided, that nofCells is less then the
        %   number of all available cells)        
        %   CommonHandles: containing the training data and the
        %       selected classifier, the ACC basic variable.
        %   
        %   OUTPUT:
        %   plate,number: integer arrays identifying the proposed
        %   cells. They have nofCells elements each. If there are no more
        %   possible cells to propose, is must be filled out with zeros.
        %   image: string cellarray identifying the selected cell. The
        %   image is the one with the contours. (anal3)
        [plate,image,cellNumber] = proposeNextCell(obj,nofCells,CommonHandles)        
        
        %getParameterValues
        % This functions returns the actual parameters of the active
        % learner in a similar format to getParameters function.
        %   OUTPUT:
        %      paramarray a cellarray with the parameter values. The order 
        %      of the parameters is the same as in the
        %      getParameters function.
        paramarray = getParameterValues(obj);
    end
    
    methods (Abstract,Static)
        %getParameters method
        % OUTPUT parameters
        %   a cellarrray with structures. Every structure describes one
        %   input parameter for the constructor of that class. The order in
        %   the cellarray must be the same as in the constructor's
        %   parameter list. This structure has the following fields:
        %       .name: the name of the parameter (this will be printed to the user when asked for the parameter value on a GUI)
        %       .type: int/enum/str right now. This determines the type
        %       of the parameter. If it is enum, then the enum values are
        %       listed in the next field.
        %       .values: if the type is enum, then the possible values are
        %       listed here in another cellarray.
        paramarray = getParameters();
    end
    
    methods (Static)
        %checkParameters method:
        %  Checks for a parameter array wheter it is a valid constructor
        %  parameter list in the form described above. It has a default
        %  constant true implementation but it can be overwritten in the
        %  child classes.
        %  INPUT: paramcell: the parameters to check
        %  OUTPUTS: bool: a boolean integer indicating the validness
        %           msg: an error msg which will be displayed to the user
        %           if the parameters are not correct.
        %  We note that the int typed parameters are checked to be numbers
        %  (can be double also!), and the ENUMS can get only values which
        %  are listed in teh values array. Any other checks can be
        %  implemented here.
        function [bool,msg] = checkParameters(~)
            bool = 1;
            msg = '';
        end
    end
    
end

