classdef ALController < handle
    %ALController The controller class of the full AL process.
    %   This class is implementing the AL process in a highly abstract way.
    %   The data model and the predictors and the AL algorithms are
    %   meeting the requirements of the interfaces, that's why they can be
    %   changed.    
    %   Even with this high abstraction level there can be cases when the
    %   AL cylce is used differently (e.g. run a test experiment, or a real
    %   active learning scenario, demos etc.) for which this class contains
    %   several functions.
    
    properties
        % Used objects:
        predictor
        predictorStore
        labeling
        data
        alAlgorithm
        errorMeasurement
        plots

        %Used arrays, and constants
        nofStartCells
        nofNextCells
        numberOfIteration
        trainingSet
        testSet
        randomProfile
        randomSeed
                
        %the predicted positions is a 3D array which stores all the
        %predictions for the objects in every AL cycle so that movement,
        %and dinamism of an object can be followed. The first 2 dimension
        %is described in the same way as one output of the
        %predictor.predict function (columns are the observations, 1st
        %dimension is output dimension the 2nd is the size of labeling
        %pool). The 3rd dimension is time.
        predictedPositions
        
       %Errors made on test set.
       testError % For sinlge runs.
       initialTestError % Inital error for single run
       testErrorPerDimension;
       initialTestErrorPerDimension
       
       uncertaintyByPredictor
       initialUncertaintyByPredictor;       
       uncertaintyByCovariance;
       initialUncertaintyByCovariance;
       testUncertaintyByCovariance;
       initialTestUncertaintyByCovariance;
            
       
       %RESULT FIELDS FOR MULTIPLE RUNS
       
       % If we run multiple experiments it contains the errors under each other.
       multipleTestError
       multipleTestErrorPerDimension;       
       multipleUncertaintyByPredictor;       
       multipleUncertaintyByCovariance;
       multipleTestUncertaintyByCovariance;       
       % The multiple inital error
       multipleInitialTestError
       multipleInitialTestErrorPerDimension
       multipleInitialUncertaintyByPredictor;
       multipleInitialUncertaintyByCovariance;
       multipleInitialTestUncertaintyByCovariance;       
       % The multiple training set, to know on which objects the training
       % was done.
       multipleTrainingSet   
       %multiple predicted positions
       multiplePredictedPositions
       
       %EXTRA FIELDS FOR ACC USE
       %These fields are not initalized in the constructor but they can be
       %set by another program. These fields must be set before use
       %proposeNextCellACC function                    
       SALT
       classifierName
       
       
    end
    
    methods        
        function obj = ALController(varargin)
        %Constructor. The varargin must contain the following parameters:
        %   1, the chosen predictor as string - currently implemented:
        %       - GPML (0 parameter)
        %       - NeuralNetwork (4 parameter BURNT IN!)
        %   2, the selected Data and corresponding labeling as number -
        %   types:
        %       0 Empty data: it will be set later by program (0 parameter)
        %       1 CommonHandlesData: to simulate AL cylce (2 parameter:
        %       CommonHandles, classIdx)
        %       2 ArtificialCellData: use artificial data (1 parameter: the
        %       previously generated Aritifical Cell Data OR the number of
        %       cells to be generated in a new ArtificialCellData. You can
        %       change by the comments in the code at data class
        %       constructors switch case 2.) 
        %       3 GreyCirclesData: very simple grey (white-black) circles
        %       (1 parameter: number of the grey circles)
        %       4 SIMCEPData: use artificial data generated by SIMCEP (1
        %       parameter: the previously generated SIMCEPData) 
        %   (here comes the parameters for the Data if there is any)
        %   3, Active Learning Algorithm by string - types
        %       - RandomChoice (0 parameter)
        %       - BiggestVariance (1 parameter: the forgetting type which can be: LINEAR, NONE, 10WINDOW, 'EXPONENTIAL')
        %       - OutOfBounds (1 parameter BURNT IN the bounds for the
        %       regression plane [XLow XUp; YLow YUp] format. The
        %       information is extracted from the data which must store the
        %       info.) Be careful if you use empty data, then [0 1000; 0
        %       1000] used for bounds as default. Later you can change this
        %       but you have to modify also the bounds for the data and for
        %       the ALalgorithm OutOfBounds.
        %       - GreatestDistance (0 parameter)
        %       - EmptyRegions2D (0 parameter)
        %       - UncertaintyBased (1 parameter: the dimension combination type: 'MEAN','SUM','MAX','MIN','MUL')
        %       - QueryByCommittee (1 parameter: the number of committee members. )
        %       - OverAllUncertaintySampling (0 parameter /Actually there is one parameter: the error measurement object, but that parameter giving is handled by the Controller/)
        %   4, in a cellarray the parameters for the
        %   ActiveLearningAlgorithm. If there is no parameter then an empty
        %   cellarray.
        %   1-4, points are compulsary , the last 2 one is optional.        
        %   5, The size of initial training set
        %   6, The number of next cells.            
            
            obj.predictorStore = PredictorStore();   
            obj.errorMeasurement = ErrorMeasurement();            
            obj.trainingSet = [];
            obj.numberOfIteration = 0;
            obj.plots = Plots();
            
            %this is for clearing up previous global variables.
            clear global myInitialization;
                        
            % ********** START OF OPTIONS PART
            
            % ***** Predictors *****
            
            currentArgumentum = 1;
            
            if strcmp(varargin{currentArgumentum},'GPML')                
                    obj.predictor = GPMLPredictor();                     
            elseif strcmp(varargin{currentArgumentum},'NeuralNetwork')
                    obj.predictor = NNPredictor(20,0.7,0.2,0.1);
            end
            currentArgumentum = currentArgumentum + 1;
            
            % ***** Data class constuctors *****
            
            switch varargin{currentArgumentum}
                case 0
                    currentArgumentum = currentArgumentum+1;
                case 1
                    obj.data = CommonHandlesData(varargin{currentArgumentum+1},varargin{currentArgumentum+2});
                    obj.labeling = CommonHandlesLabeling(varargin{currentArgumentum+1},varargin{currentArgumentum+2});
                    currentArgumentum = currentArgumentum +3;
                case 2            
                    obj.data = varargin{currentArgumentum+1};%ArtificialCellsData(varargin{currentArgumentum+1});%
                    obj.labeling = ArtificialCellsLabeling(varargin{currentArgumentum+1});%ArtificialCellsLabeling(obj.data);%
                    currentArgumentum = currentArgumentum +2;
                case 3            
                    obj.data = GreyCirclesData(varargin{currentArgumentum+1});
                    obj.labeling = GreyCirclesLabeling();
                    currentArgumentum = currentArgumentum +2;
                case 4
                    obj.data = varargin{currentArgumentum+1};
                    obj.labeling = SIMCEPLabeling(obj.data);
                    currentArgumentum = currentArgumentum +2;                    
            end
            
            % ***** AL Algorithm constructors *****
            if strcmp(varargin{currentArgumentum},'RandomChoice')
                currentArgumentum = currentArgumentum+1;
                obj.alAlgorithm = RandomChoice();                  
            elseif strcmp(varargin{currentArgumentum},'BiggestVariance')                                                 
                currentArgumentum = currentArgumentum+1;
                obj.alAlgorithm = BiggestVariance(varargin{currentArgumentum});                    
            elseif strcmp(varargin{currentArgumentum},'OutOfBounds')
                currentArgumentum = currentArgumentum+1;
                if isempty(obj.data)
                    obj.alAlgorithm = OutOfBounds({'[0 1000; 0 1000]'});
                else
                    obj.alAlgorithm = OutOfBounds({obj.data.bounds});
                end
            elseif strcmp(varargin{currentArgumentum},'GreatestDistance')
                currentArgumentum = currentArgumentum+1;
                obj.alAlgorithm = GreatestDistance();                
            elseif strcmp(varargin{currentArgumentum}, 'EmptyRegions2D')                    
                currentArgumentum = currentArgumentum+1;
                obj.alAlgorithm = EmptyRegions2D();                
            elseif strcmp(varargin{currentArgumentum},'UncertaintyBased')
                currentArgumentum = currentArgumentum+1;
                obj.alAlgorithm = UncertaintyBased(varargin{currentArgumentum}); 
            elseif strcmp(varargin{currentArgumentum},'QueryByCommittee')
                currentArgumentum = currentArgumentum+1;
                obj.alAlgorithm = QueryByCommittee(varargin{currentArgumentum});
            elseif strcmp(varargin{currentArgumentum},'OverAllUncertaintySampling')
                currentArgumentum = currentArgumentum+1;
                obj.alAlgorithm = OverAllUncertaintySampling(obj.errorMeasurement);
            end             
            currentArgumentum = currentArgumentum+1;
            
            
            % ***** AL other options *****
            
            if (nargin == currentArgumentum+1)            
                obj.nofNextCells = varargin{currentArgumentum+1};               
                obj.nofStartCells = varargin{currentArgumentum};                               
            elseif (nargin == currentArgumentum)
                obj.nofNextCells = 555;
                obj.nofStartCells = varargin{currentArgumentum};                               
            else                    
                obj.nofStartCells = 5;               
                obj.nofNextCells = 555;
            end
                        
            % END of options part **********                        
            
            %provide the predictor with the data
            obj.predictor.databounds = obj.data.bounds;
                        
        end
                
        function runExperiment(obj, numberOfIteration)
        % The runExperiment function.
        % Run experiment is a function which runs an active learning cycle
        % based on a strict schedule. This function requires a test data to
        % measure the AL performance and then it supposes that we label
        % exactly the required next cell.                    
            
            % That means, we run statistical experiment, with more than one
            % iteration. The results will be in the multiple fields.
            if nargin>1
               
               %make saving directory                              
               if exist(['..' filesep 'lastExperimentResults'],'dir')
                   warning('The full ../lastExperimentResults folder will be emptied!');
                   %getString = input('Do you want to continue? Y/N\n','s');
                   %if getString=='N'
                   %   return;
                   %end
                   rmdir(['..' filesep 'lastExperimentResults'],'s');
               end
               mkdir(['..' filesep 'lastExperimentResults']);                                             
                                              
               obj.numberOfIteration = numberOfIteration;
               disp('Multiple AL runs.');
               
               %set random seed
               obj.randomProfile = 'default';
               obj.randomSeed = 19940919;
               rng(obj.randomProfile);
               rng(obj.randomSeed);                              
               
               obj.logger();
               
               %Initialize multiple fields.
               obj.multipleTestError = zeros(numberOfIteration,obj.nofNextCells);
               obj.multipleInitialTestError = zeros(numberOfIteration,1);
               obj.multipleUncertaintyByPredictor = zeros(numberOfIteration,obj.nofNextCells);
               obj.multipleInitialUncertaintyByPredictor = zeros(numberOfIteration,1);
               obj.multipleUncertaintyByCovariance = zeros(numberOfIteration,obj.nofNextCells);
               obj.multipleInitialUncertaintyByCovariance = zeros(numberOfIteration,1);
               obj.multipleTestUncertaintyByCovariance = zeros(numberOfIteration,obj.nofNextCells);
               obj.multipleInitialTestUncertaintyByCovariance = zeros(numberOfIteration,1);
             
               obj.multipleTrainingSet = zeros(numberOfIteration,obj.nofStartCells+obj.nofNextCells);               
               obj.multiplePredictedPositions = cell(1,numberOfIteration);
               
               
               obj.alAlgorithm.reset();
               
               for i=1:numberOfIteration
                   obj.logger('**********\n%d. run\n**********\nStarted at: ',i);                   
                   
                   fprintf('**********\n%d. run\n**********\n',i);
                   obj.logdate();
                   %recursive call with less argument
                   obj.runExperiment();
                   
                   obj.multipleTestError(i,:) = convert2RowVector(obj.testError);
                   obj.multipleInitialTestError(i) = obj.initialTestError;                                      
                   
                   obj.multipleUncertaintyByPredictor(i,:) = convert2RowVector(obj.uncertaintyByPredictor);
                   obj.multipleInitialUncertaintyByPredictor(i) = obj.initialUncertaintyByPredictor;
                   
                   obj.multipleUncertaintyByCovariance(i,:) = convert2RowVector(obj.uncertaintyByCovariance);
                   obj.multipleInitialUncertaintyByCovariance(i,:) = obj.initialUncertaintyByCovariance;
                   
                   obj.multipleTestUncertaintyByCovariance(i,:) = convert2RowVector(obj.testUncertaintyByCovariance);
                   obj.multipleInitialTestUncertaintyByCovariance(i,:) = obj.initialTestUncertaintyByCovariance;
                   
                   
                   obj.multipleTrainingSet(i,:) = convert2RowVector(obj.trainingSet);                   
                   obj.multiplePredictedPositions{i} = obj.predictedPositions;
                   
                   timestamp = clock;                                                                        
                   
                   %do final saves. These variables needs less space than the
                   %big array and used mostly for evaluation.
                   mTE = obj.multipleTestError; %#ok<NASGU> it is neccessary to copy one field out of the obj, because we want to save it.
                   save(['..' filesep 'lastExperimentResults' filesep num2str(timestamp(1)) num2str(timestamp(2),'%02d') num2str(timestamp(3),'%02d') num2str(timestamp(4),'%02d') num2str(timestamp(5),'%02d') 'mTE'],'mTE','-v7.3');
                   mITE = obj.multipleInitialTestError; %#ok<NASGU>
                   save(['..' filesep 'lastExperimentResults' filesep num2str(timestamp(1)) num2str(timestamp(2),'%02d') num2str(timestamp(3),'%02d') num2str(timestamp(4),'%02d') num2str(timestamp(5),'%02d') 'mITE'],'mITE','-v7.3');
                   mUBP = obj.multipleUncertaintyByPredictor; %#ok<NASGU>
                   save(['..' filesep 'lastExperimentResults' filesep num2str(timestamp(1)) num2str(timestamp(2),'%02d') num2str(timestamp(3),'%02d') num2str(timestamp(4),'%02d') num2str(timestamp(5),'%02d') 'mUBP'],'mUBP','-v7.3');
                   mIUBP = obj.multipleInitialUncertaintyByPredictor; %#ok<NASGU>
                   save(['..' filesep 'lastExperimentResults' filesep num2str(timestamp(1)) num2str(timestamp(2),'%02d') num2str(timestamp(3),'%02d') num2str(timestamp(4),'%02d') num2str(timestamp(5),'%02d') 'mIUBP'],'mIUBP','-v7.3');
                   mUBC = obj.multipleUncertaintyByCovariance; %#ok<NASGU>
                   save(['..' filesep 'lastExperimentResults' filesep num2str(timestamp(1)) num2str(timestamp(2),'%02d') num2str(timestamp(3),'%02d') num2str(timestamp(4),'%02d') num2str(timestamp(5),'%02d') 'mUBC'],'mUBC','-v7.3');
                   mIUBC = obj.multipleInitialUncertaintyByCovariance; %#ok<NASGU>
                   save(['..' filesep 'lastExperimentResults' filesep num2str(timestamp(1)) num2str(timestamp(2),'%02d') num2str(timestamp(3),'%02d') num2str(timestamp(4),'%02d') num2str(timestamp(5),'%02d') 'mIUBC'],'mIUBC','-v7.3');
                   mTUBC = obj.multipleTestUncertaintyByCovariance; %#ok<NASGU>
                   save(['..' filesep 'lastExperimentResults' filesep num2str(timestamp(1)) num2str(timestamp(2),'%02d') num2str(timestamp(3),'%02d') num2str(timestamp(4),'%02d') num2str(timestamp(5),'%02d') 'mTUBC'],'mTUBC','-v7.3');
                   mITUBC = obj.multipleInitialTestUncertaintyByCovariance; %#ok<NASGU>
                   save(['..' filesep 'lastExperimentResults' filesep num2str(timestamp(1)) num2str(timestamp(2),'%02d') num2str(timestamp(3),'%02d') num2str(timestamp(4),'%02d') num2str(timestamp(5),'%02d') 'mITUBC'],'mITUBC','-v7.3');
                   mTS = obj.multipleTrainingSet; %#ok<NASGU>
                   save(['..' filesep 'lastExperimentResults' filesep num2str(timestamp(1)) num2str(timestamp(2),'%02d') num2str(timestamp(3),'%02d') num2str(timestamp(4),'%02d') num2str(timestamp(5),'%02d') 'mTS'],'mTS','-v7.3');
                   mPS = obj.multiplePredictedPositions; %#ok<NASGU>
                   save(['..' filesep 'lastExperimentResults' filesep num2str(timestamp(1)) num2str(timestamp(2),'%02d') num2str(timestamp(3),'%02d') num2str(timestamp(4),'%02d') num2str(timestamp(5),'%02d') 'mPS'],'mPS','-v7.3');

               end 
               
               %Do the final plots.
               obj.plots.finalPlots(obj);
               
               obj.logger('\nThe end of the experiment is%s:','');
               
               %save(['..' filesep 'lastExperimentResults' filesep num2str(timestamp(1)) num2str(timestamp(2),'%02d') num2str(timestamp(3),'%02d') num2str(timestamp(4),'%02d') num2str(timestamp(5),'%02d') '_' obj.alAlgorithm.name],'obj','-v7.3');
               
            %One simple AL cycle. (This is what the multiple one calls recursively)
            else
               
               % ***** Initialization
               
               %Zeroing traning set and average error. (also memory allocating)
               obj.trainingSet = [];
               obj.predictedPositions = [];
               obj.testError = zeros(1,obj.nofNextCells);
               obj.initialTestError = 0;
               obj.uncertaintyByPredictor = zeros(1,obj.nofNextCells);
               obj.initialUncertaintyByPredictor = 0;
               obj.uncertaintyByCovariance = zeros(1,obj.nofNextCells);
               obj.initialUncertaintyByCovariance = 0;
               obj.testUncertaintyByCovariance = zeros(1,obj.nofNextCells);               
               obj.initialTestUncertaintyByCovariance = 0;           
			   
			   %reset Al algorithm to start it in clean state
			   obj.alAlgorithm.reset();
			   
                   
               %determine test Set
               fprintf('Test set determining.\n');
               obj.testSet = determineTestSet(obj.data.sizeOfLabelingPool);
               for i=1:length(obj.testSet)                   
                   obj.data = obj.data.label( obj.testSet(i) , obj.labeling.label(obj.testSet(i),obj.data));
               end
               
               %clear out occasionally remaining global variables
               clearvars -global myInitialization;
               
               % END of initialization *****
               
               %Choose randomly few cells to start AL cylce.
               fprintf('Random few cells to start AL cycle.\n');
               for i=1:obj.nofStartCells
                   ranks = randperm(obj.data.sizeOfLabelingPool);
                   obj.trainingSet(end+1) = chooseByRules(ranks,obj.trainingSet, obj.testSet);
                   obj.data = obj.data.label( obj.trainingSet(end) , obj.labeling.label(obj.trainingSet(end),obj.data));
               end

               %The first training for evaluating the errors on the inital
               %training set.
               fprintf('AL inital error measurement.\n');               
               [obj.initialTestError,obj.initialTestErrorPerDimension] = obj.errorMeasurement.testValidate(obj.predictor, obj.predictorStore, obj.data, obj.trainingSet, obj.testSet);
               if (obj.predictor.isDistribution)
                   [obj.initialUncertaintyByPredictor] = obj.errorMeasurement.totalCovarianceFromPredictor(obj.predictor, obj.predictorStore, obj.data,obj.trainingSet);
                   if isa(obj.predictor,'GPMLPredictor')
                       [obj.initialUncertaintyByCovariance] = obj.errorMeasurement.calculateOverAllUncertainty(obj.predictor, obj.predictorStore, obj.data,obj.trainingSet,[]);
                       [obj.initialTestUncertaintyByCovariance] = obj.errorMeasurement.calculateOverAllUncertainty(obj.predictor, obj.predictorStore, obj.data,obj.trainingSet,[],obj.testSet);
                   end
               end
                              
               
               fprintf('--------\n');
               
               %**** The huge AL-cycle ****
               i=1;
               while i<=obj.nofNextCells
                  fprintf('AL cycle: %d. cell\n',i);
                  obj.logdate();
                  obj.logger('AL cycle: %d. cell starts at: ',i);
                  %Choosing next cell
                  [ranking]= obj.alAlgorithm.nextCell(obj.predictor,obj.predictorStore,obj.data,obj.trainingSet, obj.testSet);

                  %Labeling them
                  newSamples = chooseByRules(ranking, obj.trainingSet, obj.testSet);
                  for j=1:length(newSamples)
                      obj.data = obj.data.label( newSamples(j) , obj.labeling.label(newSamples(j),obj.data));
                      obj.trainingSet = [obj.trainingSet, newSamples];
                  end                  
                  
                  %Extra activities to evaluate performance:
                  
                  %Error measurement
                  fprintf('Error measurements:\n');
                  [obj.testError(i),obj.testErrorPerDimension(:,i)] = obj.errorMeasurement.testValidate(obj.predictor, obj.predictorStore, obj.data, obj.trainingSet, obj.testSet);
                  if (obj.predictor.isDistribution)
                      [obj.uncertaintyByPredictor(i)] = obj.errorMeasurement.totalCovarianceFromPredictor(obj.predictor, obj.predictorStore, obj.data,obj.trainingSet);
                      if isa(obj.predictor,'GPMLPredictor')
                          [obj.uncertaintyByCovariance(i)] = obj.errorMeasurement.calculateOverAllUncertainty(obj.predictor, obj.predictorStore, obj.data,obj.trainingSet,[]); 
                          [obj.testUncertaintyByCovariance(i)] = obj.errorMeasurement.calculateOverAllUncertainty(obj.predictor, obj.predictorStore, obj.data,obj.trainingSet,[],obj.testSet);
                      end
                  end
                  obj.logger('Test error:\n Test: %f\n Errors finished at: ',obj.testError(i));
                                         
                  %do prediction place storing
                  [readyPredictor] = getPredictorMacro(obj.predictor, obj.data, obj.predictorStore, obj.trainingSet);
                  allInputs = obj.data.getData();
                  if size(obj.predictedPositions,2)<2
                    obj.predictedPositions(:,:,1) = readyPredictor{1}.predict(allInputs);
                  else
                    obj.predictedPositions(:,:,end+1) = readyPredictor{1}.predict(allInputs);
                  end
                  
                  %do plots
                  obj.plots.doPlots(obj,i);                   
                                    
                  i=i+j;                                    
                  
                  fprintf('--------\n');
               end
               %**** END of AL Cycle ****
                                             
               obj.plots = obj.plots.setAll2Value(0);
               
            end            
        end
                
        function obj = runDemo(obj)                                          
        % The runDemo
        % This function can be used to test the capabilites of our chosen
        % active learner and see by eyes the results. This does not make a
        % test set, it's only training.
            
           % ***** Initialization

           %Zeroing traning set and average error.
           obj.trainingSet = [];
           obj.testError = zeros(1,obj.nofNextCells);
           obj.initialTestError = 0;          
           obj.testSet = [];           
           
           % delete plots as we don't need them
           obj.plots.closeall();

           % END of initialization *****

           %Choose randomly few cells to start AL cylce.
           fprintf('Random few cells to start AL cycle.\n');
           for i=1:obj.nofStartCells
               ranks = randperm(obj.data.sizeOfLabelingPool);
               obj.trainingSet(end+1) = chooseByRules(ranks,obj.trainingSet, obj.testSet);
               obj.data = obj.data.label( obj.trainingSet(end) , obj.labeling.label(obj.trainingSet(end),obj.data));
           end
          

           %The huge AL-cycle
           i=1;
           while i<=obj.nofNextCells
              fprintf('AL cycle: %d. cell\n',i);
              %Choosing next cell
              [ranking]= obj.alAlgorithm.nextCell(obj.predictor,obj.predictorStore,obj.data,obj.trainingSet,obj.testSet);

              %Labeling them
              newSamples = chooseByRules(ranking, obj.trainingSet, obj.testSet);
              for j=1:length(newSamples)
                  obj.data = obj.data.label( newSamples(j) , obj.labeling.label(newSamples(j),obj.data));
                  obj.trainingSet = [obj.trainingSet, newSamples];
              end

              %Error measurement
              %[obj.testError(i),obj.predictorStore] = obj.errorMeasurement.testValidate(obj.predictor, obj.predictorStore, obj.data, obj.trainingSet, obj.testSet);
              
              i=i+j;
              %
              getString = input('Do you want to continue teaching? Y/N\n','s');
              if getString=='N'
                  break;
              end
              %
              % fprintf('Trainingset:\n'); commented out it's unnecessary
              % disp(obj.trainingSet);
              fprintf('--------\n');
           end               
       

            %
            %Here we'll plot all the predictions.
            [ trainedPredictor] = getPredictorMacro( obj.predictor,obj.data, obj.predictorStore, obj.trainingSet);
            toPredictInputs = obj.data.getData();
            targets = trainedPredictor{1}.predict(toPredictInputs);
            obj.data.plotDataToRegressionPlane(1:length(toPredictInputs),targets);
        end
                        
        function [CommonHandles] = proposeNextCellACC(obj,CommonHandles)
        %this function implements the active learning in ACC. From the
        %input CommonHandles we'll use the sampling pool to build up a data
        %model for this AL scenario. Then we use the obj.SALT to classify
        %the cells in the samplingPool.
        %CURRENTLY WORKS ONLY WITH SALT!        
            
            %clear out predictor store because the training set is very
            %unstable it can change very fast.
            obj.predictorStore = PredictorStore();
            %we don't use test set currently
            obj.testSet = [];   
            
            CommonHandles = trainSALTForAC(CommonHandles);
            features = zeros(size(CommonHandles.SamplingPool.Features, 1),CommonHandles.featureDimension-2);
            for i=1:size(CommonHandles.SamplingPool.Features, 1)
                features(i,:) = CommonHandles.SamplingPool.Features(i, :);    
            end;
            
            %SALT specific region
            obj.SALT.trainingData = convertACC2SALT(CommonHandles);
            obj.SALT.trainingData.instances = features;
            obj.SALT.trainingData.labels = ones(size(features, 1));
            [out,~] = sacPredict(CommonHandles.AC.SALT.model, obj.SALT.trainingData);
            CommonHandles.SamplingPool.Predictions = convert2RowVector(out);
            %END
            
            %Currently we propose cells only for one regression class which
            %is the first among the classes.
            
            [~,selectedRegressionClass] = max(CommonHandles.RegressionClass);
            obj.data = ACCData(CommonHandles,selectedRegressionClass);
            obj.trainingSet = obj.data.realLabels;
            
            [ranking]= obj.alAlgorithm.nextCell(obj.predictor,obj.predictorStore,obj.data,obj.trainingSet,obj.testSet);
            
            %we choose a cell which is not in the training set. (the test set is empty)
            newSamples = chooseByRules(ranking, obj.trainingSet, obj.testSet);
            
            [CommonHandles.SelectedImageName,CommonHandles.SelectedCell] = obj.data.map2ACC(newSamples(1));            
            
            CommonHandles.AC = obj;
            
            %{
            TODO:                         
                  HANDLE extreme cases when there is no cell in the
                  sampling pool, when there is less cell in the reg plane
                  than it required and so on...
            %}
            
        end
        
        function delete(obj)
            obj.plots.closeall();            
        end
        
        function logger(obj,text,args)
        %logger function collects all the log information into a log file
        %which place is fixed: ../lastExperimentResults/explog.txt.
        % If it fails to write the file then is prints to the stdout.
        %This function has 2 types to use (they can be called with 0 or 1
        %parameter):
        %   0 param: basic log ususally at the start of the experiment.
        %   1 param: a timestamp with the text written before the time.
            try
                if nargin<2
                   %LOG to file
                   f = fopen(['..' filesep 'lastExperimentResults' filesep 'explog.txt'],'w');                   
                   logToFileBasic(f);
                   fclose(f);
                   %END of LOG
                else               
                   f = fopen(['..' filesep 'lastExperimentResults' filesep 'explog.txt'],'a');
                   fprintf(f,text,args);
                   obj.logdate(f);
                   fclose(f);
                end
            catch % do the same but to stdout!
                if nargin<2                   
                   f = 1;                   
                   logToFileBasic(f);                   
                   %END of LOG
                else               
                   f = 1;
                   fprintf(f,text,args);
                   obj.logdate(f);                   
                end
            end
            
                function logToFileBasic(f)
                   fprintf(f,'----------------\nEXPERIMENT LOG FILE\n----------------\n\n');                                             
                   fprintf(f,'The start of the experiment is: ');
                   obj.logdate(f);
                   fprintf(f,['The used random seed is: ' num2str(obj.randomSeed), ' and it is used with ' obj.randomProfile ' profile.\n']);               
                   fprintf(f,['Number of independent runs: ' num2str(obj.numberOfIteration) '\n']);
                   fprintf(f,['Size of initial set: ' num2str(obj.nofStartCells) '\n']);
                   fprintf(f,['Number of new objects: ' num2str(obj.nofNextCells) '\n\n']);               
                   fprintf(f,['Used active learning algorithm: ' obj.alAlgorithm.name() '\n\n']);
                   fprintf(f,['Used data: ' class(obj.data) ' with ' num2str(obj.data.sizeOfLabelingPool) ' objects (cells).\n']);
                   fprintf(f,['Theoretical bounds: ' obj.data.bounds '\n\n']);
                   fprintf(f,'Used regressor details:\n');
                   obj.predictor.print2file(f); 
                   fprintf(f,'\n');
                end
        end                 
        
        function logdate(~,file)
        %logdate function prints out the date information into the file or
        %if it's not given then to the standard output. Be careful it puts
        %endline only to the end and not before.
            c = clock;
            if nargin<2
                fprintf([num2str(c(1),'%04d') '.' num2str(c(2),'%02d') '.' num2str(c(3),'%02d') '.' num2str(c(4),'%02d') ':' num2str(c(5),'%02d') '.\n']);
            else
                fprintf(file,[num2str(c(1),'%04d') '.' num2str(c(2),'%02d') '.' num2str(c(3),'%02d') '.' num2str(c(4),'%02d') ':' num2str(c(5),'%02d') '.\n']);
            end
        end
    end
    
end

