classdef ALalgorithm < handle
    %ALALGORITHM Interface for our Active Learning methods.
    %   An Active learning method is basically a sample choosing algorithm,
    %   which tries to select the best next training sample from the
    %   iterative cycle of active learning.
    %   An AL algorithm must have a NextCell method, which tells us the
    %   next cell to choose from the unlabeled set. (This will be the index
    %   of that cell, as described in the Data interface all data must be
    %   able to identify its samples by numbers)
    
    properties                        
    end
    
    methods (Abstract)
        
        %The nextCell method.
        % INPUT parameters:
        %   predictor: a brand new predictor which can be trained if the
        %   appropriate predictor is not found in the predictor store.
        %   predictorStore: a container of the previously trained
        %   predictors.
        %   data: the data object which contains the labels and inputs.        
        %   trainingSet: array of indeces which objects we used to train
        %   previous predictor. We are allowed to train predictor only from
        %   this set. There are other labeled objects in data but they are
        %   reserved for testing our method.
        %   testSet: the indeces of other labeled objects which cannot be
        %   used for training. (not neccessary to use)
        %   
        % OUTPUT parameters:
        %   nextCell: an array of indeces in ranking order. The first cell
        %   is the most informative according to the ALalgorithm and so on.
        %   predictorStore: if the ALalgorithm needed new predictor to
        %   train than it was stored in the predictorStore and we give back
        %   for further usage.
        [nextCell] = nextCell(obj,predictor,predictorStore,data,trainingSet,testSet)
        
        %The reset method
        %   This is a reinitialization method, for AL algorithms. If we
        %   want to start a totally new cylce with the same algorithm we do
        %   not need to construct another one, but we can use this function
        %   to reset all the parameters in the algorithm.
        reset(obj)
        
        %This method is quite similar to the toString method but it gets
        %the class's name, with the most important extra features.
        [string] = name(obj)
        
        %getParameterValues
        % This functions returns the actual parameters of the active
        % learner in a similar format to getParameters function.
        %   OUTPUT:
        %      paramarray a cellarray with the parameter values. The order 
        %      of the parameters is the same as in the
        %      getParameters function.
        paramarray = getParameterValues(obj);
    end    
    
    methods (Abstract, Static)
        %getParameters method
        % OUTPUT parameters
        %   a cellarrray with structures. Every structure describes one
        %   input parameter for the constructor of that class. The order in
        %   the cellarray must be the same as in the constructor's
        %   parameter list. This structure has the following fields:
        %       .name: the name of the parameter (this will be printed to the user when asked for the parameter value on a GUI)
        %       .type: int/enum/str right now. This determines the type
        %       of the parameter. If it is enum, then the enum values are
        %       listed in the next field.
        %       .values: if the type is enum, then the possible values are
        %       listed here in another cellarray.
        paramarray = getParameters();
    end
    
    methods (Static)
        %checkParameters method:
        %  Checks for a parameter array wheter it is a valid constructor
        %  parameter list in the form described above. It has a default
        %  constant true implementation but it can be overwritten in the
        %  child classes.
        %  INPUT: paramcell: the parameters to check
        %  OUTPUTS: bool: a boolean integer indicating the validness
        %           msg: an error msg which will be displayed to the user
        %           if the parameters are not correct.
        %  We note that the int typed parameters are checked to be numbers
        %  (can be double also!), and the ENUMS can get only values which
        %  are listed in teh values array. Any other checks can be
        %  implemented here.
        function [bool,msg] = checkParameters(~)
            bool = 1;
            msg = '';
        end
    end
    
end

