classdef (Abstract) Predictor
    %Predictor Interface for all predictors we use in our AL strategies
    %   The Predictor class is abstract, it much more like an interface in fact.
    %   All the methods in it must be implemented for different predictors
    %   just like: GaussianProcesses or NeuralNetworks and so on.
    %       What we require from 'data' is:
    %         To be able to be trained with exmaples, and after training
    %         be able to make predictions for new samples.
    %       The corresponding training exmaples must be in the same size,
    %       but training examples in different training processes can be
    %       different. This regulation is also applicable to the
    %       corresponding targets. (they do have to match with each other
    %       of course, not with the inputs (features))
    %       Usually inputs, and targets are n by 1 dimensional. (One of
    %       them, and an input set consists of horizontal concatenation of
    %       them)
    
    properties
        %We need to know of all types of predictors whether they are
        %capable to predict discrete points or they can give us variances.
        %This functionality is stored in the isDistribution variable.
        isDistribution
        
        %ususally a predictor needs a model to store information
        %gained from the trainnig examples, based on which it can make the
        %predictions
        model
        
        %It can be essential to the predictor to know the theoretical
        %bounds of the data.
        databounds
    end
    
    methods (Abstract)        
        %TrainModel This function is the "interface" of the modelTrainers. Model
        %training is essential for predictions, this is the learning part.         
        %
        % FORMATS:
        %   Inputs: An array.
        %       Columns refers to observations.
        %       The i th row contains the i th feature value for all observations.
        %       ==> The first dimension of the array is the input dimension.
        %   Outputs: An array.
        %       Columns are the observations (this have to match with the second
        %       dimension of the input array).
        %
        % @ref: see also the Predictor function, which is the interface for
        % predictors.    
        obj = train(obj,inputs,targets)                   

        %PREDICTOR This function is the "interface" of the predictors. 
        % @ref: see also the TrainModel function, which is the interface for
        % training models.
        %   INPUT:
        %       The inputs: these are the new inputs, we want to have predictions
        %       of them. Format: it is the same inputs format described in
        %       TrainModel.
        %   Outputs are the single output points if the predictor is
        %   'deterministic' in a way that it does not give us
        %   probability/distribution. If the predictor is capable to predict
        %   distribution then outputs are the means, and the second output argument
        %   is the variance for the distriubtion.
        %   STANDARD OUTPUT FORMAT FOR THIS:
        %       'outputs' must be in the same format, as the targets:
        %       the columns are the observations, and the rows are the output
        %       dimensions. ==> the second dimension will match with the second
        %       dimension of inputs.
        %
        %       'variances':
        %       This must be a full covariance matrix, for each output dimension.
        %       If the output dimension is d and the number of the new samples is
        %       n, then the variances matrix is an (n*d)x(n*d) matrix. It contains
        %       non-zero elements in the diagonal (n*d) blocks.
        %       These (n*d) blocks are the covariance matrices between the new
        %       samples for each dimension. Sometimes these covariance matrices are
        %       estimated with diagonal matrices supposing that the covariance
        %       between different samples is 0. ==> only the variance values will
        %       be displayed in these cases in the diagonal.       
        [outputs,variances] = predict(obj, inputs)        
        
        %The toString function must accomplish the requirement that it
        %provides all the information of the Predictor, not only it's name
        %but all the parameters used. (mostly all type information of the
        %model, the used covariance functions, training ratios etc., but we
        %don't need the trained hyperparameter values.)
        %These properties can be long so for better format results this
        %function prints out the requested data to the provided file handle
        %which should be opened for writing or appending.
        print2file(obj,f);                                        
    end
    
end