classdef GPMLPredictor < Predictor
    %GPMLPredictor Gaussian Processes implementation of the Predictor
    %interface
    %  GPML
%      The GPML process use the gp function for predicting, but for that it
%      needs quite a lot of functions and parameters. These are:           
%           hyp: for hyperparameters
%          infExact:
%           meanfunc: mean function
%           covfunc: covariance function
%           likfunc: likelihodd function
%       To determine these last 3 functions there are lots of possibility in the
%       GPML. It is not evident which to use. The gp function do not need
%       'training', but we can optimize some parameters. 
%       At the moment I cannot see possibility to optimise the functions
%       BUT:
%       for the hyperparameters there are optimization methods, that is
%       what we'll do in the TrainModel function.
%           More fields of the model struct:
%           targetMean: the mean of the targets (required to retrieve the
%           predicted outputs to the targeted space /because the targets
%           are normalized/)
%           targetVariance: because of the same reasons as above
%           targets: the normalized targets in GPML format! Transposed.
%           inputs: the inputs, in GPML format. Not normalized.

    
    properties
        %Properties which are essential for the training.
        
        %The normalization. Check for norm values the /util/normalizeData        
        norm;
        %Mean function for GP
        meanf;
        %Covariance function for GP
        covf; 
        %Likelyhood function for GP
        likf;
    end
    
    methods        
        function obj = GPMLPredictor()        
            fprintf('You have initialized a new GPML predictor\n');
            obj.isDistribution = 1;
            obj.norm = 0;
            obj.meanf= @meanConst;%{@meanSum, {@meanLinear, @meanConst}};%@meanNN;%
            obj.covf = @covSEard;%@covSEiso;%@covNNone;%@covSEfact;%
            obj.likf = @likGauss;
            clearvars -global myInitialization myHyps;
        end
        
        function obj = train(obj, inputs, targets)
        %TRAIN It is a trainer function for the gpml.
        %   The gpml does not need trainig explicitly, but we can optimize
        %   parameters beforehand, so it can perform better results.
        %   Also many input parameters had to be chosen, which are copied
        %   here from the constructor through properties of this class.
        %   The INPUTS: inputs and targets are match the requirements written in
        %   TrainModel. (columns are the observations), but the minimize function
        %   uses the transposed format.
        %   We can decide in this code if we want to do normalization on the
        %   inputs. (on the targets we do normalization all the time)
        %   This can be set with normalization field's value (in model structure)
        %   at the start of the code. The value is not a boolean but an indicator
        %   for the normalization method. For the values meaning you can check
        %   normalizeData function in the utils.
            disp('Train with GPML-v3.5');                                                
            
            model.normalization = 0;

            if model.normalization
                [inputNormalizedTransposed,model.ranges] = normalizeData(inputs',model.normalization);
                inputs = inputNormalizedTransposed';    
            end
            
            %Normalize targets
            targetMean = mean(targets,2);
            targetVariance = std(targets,[],2);
            targets = (targets-repmat(targetMean,1,size(targets,2))) ./ repmat(targetVariance,1,size(targets,2));

            global myInitialization;
            global myHyps;

            if ~exist('myInitialization','var') || isempty(myInitialization) || myInitialization ~= 1239462,
                myHyps = cell(size(targets,1));
                for i=1:size(targets,1)
                    hyp = obj.initCovParameters(inputs,i,targetMean,targetVariance);
                    myHyps{i} = hyp;
                end
                myInitialization = 1239462;
            end

            % Defining functions.
            meanfunc = obj.meanf;
            covfunc = obj.covf; 
            likfunc = obj.likf;

            %Define some hyperparameters. (but these will be changed when we optimize)
            %hyp.cov = [1;1];
            %hyp.mean = zeros(size(inputs,1)+1,1);
            %hyp.lik = log(0.1);

           

            for i=1:size(targets,1)
                
                %model.hyp{i} = hyp;
                hyp = myHyps{i};%!!! this was previously !!!% %hyp = obj.model.hyp{i};
                % pick the best funcV from several inits.                
                %[model.hyp{i},funcV] = minimize(hyp, @gp, -500, @infExact, meanfunc, covfunc, likfunc, inputs', targets(i,:)');                    
                
                %after every 20th training sample do reinit.
                %if (mod(size(inputs,2),20)~=0)
                    newModel = minimize(hyp, @gp, -100, @infExact, meanfunc, covfunc, likfunc, inputs', targets(i,:)');
                %else
                %    hyp = obj.initCovParameters(inputs,i,targetMean,targetVariance);
                %end
                if (exist('newModel','var') && isfield(newModel,'cov') && ~isnan(newModel.cov(1)))
                   model.hyp{i} = newModel;
                else
                   model.hyp{i} = hyp; % restore previous hyps
                   clear global myInitialization; % request for initialization in next cycle.                   
                end
                
                %check whether the hyps can make a prediction
                try
                    K = feval(covfunc, model.hyp{i}.cov, inputs');
                    chol(K);
                catch
                    model.hyp{i} = obj.initCovParameters(inputs,i,targetMean,targetVariance);
                end
                
                %
                %model.hyp{i} = hyp;
                myHyps{i} = model.hyp{i};
                                
                
            end

            model.meanfunc = meanfunc;
            model.covfunc = covfunc;
            model.likfunc = likfunc;
            model.targetMean = targetMean;
            model.targetVariance = targetVariance;
            model.inputs = inputs';
            model.targets = targets';
            
            obj.model = model;
        end        
        
        function [outputs,variances] = predict(obj, inputs)
            disp('Predict with GPML.');            
            [outputs,variances] = PredictGPML(obj.model,inputs);                                    
        end
        
        function print2file(obj,f)
            fprintf(f,'GPML predictor by Rasmussen and Nickisch.\n');
            fprintf(f,['It is capable to predict distributions: ' num2str(obj.isDistribution) '\n']);
            fprintf(f,'------\n');
            fprintf(f,'Normalization for the input data:\n');
            switch obj.norm
                case 0
                    fprintf(f,'0: No normalization\n');
                case 1
                    fprintf(f,'1: (Default) each attribute is normalized to have a zero mean and 1 mean/median absolute deviation (mad)\n'); 
                case 2
                    fprintf(f,'2: (Standardization) Each attribute is normalized to fall in the range [0,1].\n');
                case 3
                    fprintf(f,'3: Each attribute is normalized to have zero mean and 1 standard deviation\n');
            end
            fprintf(f,'Used functions:\n');
            fprintf(f,['Mean function: ' func2str(obj.meanf) '\n']);
            fprintf(f,['Covariance function: ' func2str(obj.covf) '\n']);
            fprintf(f,['Likelihood function: ' func2str(obj.likf) '\n']);
        end                
        
        function [hyp] = initCovParameters(obj,inputs,i,targetMean,targetVariance)
            disp('Initialize GPML hyps.');                
            D = size(inputs,1);
            %for covSEfact    
            %hyp.cov = zeros(D*1 - 1*(1-1)/2 + 1,1);
            hyp.cov = zeros(D+1,1);
            regPlaneBounds = str2num(obj.databounds); %#ok<ST2NM> We need to use str2num because we're working with matrix and not scalar
            expectedVariance = (1/12)*(regPlaneBounds(i,2)-regPlaneBounds(i,1)).^2;
            hyp.cov(end) =  log(sqrt(expectedVariance));
            initCov = estimateInitP(inputs,min(10,size(inputs,2)-1),1);                                                                               
            hyp.cov(1:end-1) = initCov;%log(initCov);%
            hyp.mean = ((regPlaneBounds(i,2)+regPlaneBounds(i,1))/2-targetMean(i))/targetVariance(i);
            %hyp.mean = zeros(size(inputs,1)+1,1);
            hyp.lik = log(0.1);                                   
        end
    end
    
end

