function [p,lastDiff] = estimateInitP(inputs,k,maxIteration)
%estimateInitP function searching for good parameter initialization.
%    The sf parameters are the possible variances in the target space among
%    each dimension.
%    This function is written to find the p. The suggestion is that we'll try
%   as init value the average distance for similar objects. This method has a
%    parameter k.
%   Details: first we search for the k nearest neighbours (kNN) in the full input
%   space with traditional euclidean distance. Then seperately for the
%   input dimensions we calculate the mean distance for a point from its
%   kNN. Finally we take the mean over all the points. The result is an
%   array with input dimension elements which tells for every input
%   dimension what is the average of the average distance from the closest
%   points.
%   1. Find the k closest neighbours.
%   2. Meaure distances in each dimension for each cell from the k
%   corresponding closest neighbours.
%   3. Take the mean of the distances over the k closest for each cell.
%   4. take the mean over all the objects.
%
%   Unfortunately with this initialization the hyperparameters are not good
%   enough, because in the weighted disctance (when we rescale our input
%   space with the parameters), it can happen that there isn't any very
%   close point. So we do the above mentioned algorithm iteratively until
%   we see that it converge somewhere.
%
%   INPUTS:
%       inputs: inputs in format given back by the data objects (columns are observations)
%       k: the input parameter for this algorithm.
%       maxIteration (optional): if not provided then we take it as one.
%   OUTPUTS:
%       p: the estimated hyperparameters
%       lastDiff: the difference between the last 2 p in the iteration.
%%{

    if nargin<3
        maxIteration = 1;
    end
    
    p = zeros(size(inputs,1),1);
    newInputs = inputs;
    for i=1:maxIteration
        pp = p;
        p = calcP(newInputs,k);
        newInputs = diag(1./p)*inputs;
        if (i>1)            
            lastDiff = pdist([pp'; p']);
        else
            lastDiff = -1;
        end
    end    


%%}
%{
%Sanity check:
p = zeros(1,size(inputs,1))+10^-10;
p(end-2) = exp(1);
p(end-1) = exp(1);
p(end) = exp(1);
%}
end

function ell = calcP(inputs,k)
    d = pdist(inputs','euclidean');
    %S is the distance matrix
    S = squareform(d);
    %preallocate space    
    neighboursDist = zeros(size(inputs,2),k,size(inputs,1));

    %over all the objects
    for i=1:size(inputs,2)
        %Sort the distance matrix for the ith element
        [~,indices] = sort(S(i,:));    
        %over all the dimensions
        for j=1:size(inputs,1)
            %over the most closest k neighbours in total distance
            for l=2:k+1
                %store the distance in the j dimension
                neighboursDist(i,l-1,j) = (inputs(j,i)-inputs(j,indices(l))).^2;
            end        
        end
    end
    %compute the seperate mean in each dimension with euclidean pdist
    ell = (reshape(mean(mean(neighboursDist,2),1),size(inputs,1),1)).^(1/2)+10^-10;
end