function [shiftVectors,shiftPicture] = calculateShiftVectors( results,row,col)
% AUTHOR:   Abel Szkalisity
% DATE:     Aug 04, 2016
% NAME:     calculateShiftVectors
%
% This function calculates a very simple statistic about difference
% between regression predictions from different wells. There is a control
% well to which all the other wells are compared. The comparison is based
% on the most dense region of the regression plane. (e.g. where the
% majority of the cells are located). The density calculation can be
% changed by changing the appropriate function.
%
% INPUT:
%   results         A cell array sized exactly as the plate that we
%                   examine. Every cell contains a 2 by n matrix, these are
%                   the predicted positions of the specific well. From
%                   these coordinates the density is estimated.
%   row             The row index of the control well.
%   col             The column index of the control well.
%
% OUTPUT:
%   shiftVectors    A cell array sized the same as results, and every cell
%                   contains a 4 element row vector. The first 2 element of
%                   this vector is the most dense position in the control
%                   well (start point of the shift vector), the last 2
%                   element are the most dense point within the specific
%                   well (end point)
%
% COPYRIGHT
% Advanced Cell Classifier (ACC) Toolbox. All rights reserved.
% Copyright (C) 2016 Peter Horvath,
% Synthetic and System Biology Unit, Hungarian Academy of Sciences,
% Biological Research Center, Szeged, Hungary; Institute for Molecular
% Medicine Finland, University of Helsinki, Helsinki, Finland.

[rowNum,colNum] = size(results);

%Regression plane resolution in the density estimation
x1 = 0:10:1000;
x2 = 0:10:1000;
shiftPicture = zeros(length(x2)*rowNum,length(x1)*colNum);

%densityInControl flipped regarding to the first coordinate
densityInControl = densityEstimationByGaussians(results{row,col},x1,x2);
%the max function also starts from the upper left corner which is correct
%because the densityInControl picture is flipped so finally the controlY
%position will be in the original (Cartesian coordinate system)
[~,idx] = max(densityInControl(:));
controlY = mod(idx-1,length(x2))+1;
controlX = idivide(int32(idx-1),int32(length(x2)))+1;

shiftVectors = cell(size(results));

waitBarHandle = waitbar(0,'Density estimation...');

for i=1:rowNum
    for j=1:colNum        
        if ~isempty(results{i,j})
            densityInWell = densityEstimationByGaussians(results{i,j},x1,x2);
            [~,idx] = max(densityInWell(:));
            wellY = mod(idx-1,length(x2))+1;
            wellX = idivide(int32(idx-1),int32(length(x2)))+1;
            shiftVectors{i,j} = double([controlX-1 controlY-1 wellX-1 wellY-1]);
            shiftVectors{i,j} = shiftVectors{i,j}./(length(x1)-1);
        else
            %in this case the vector should not point anywhere
            densityInWell = densityInControl;
            shiftVectors{i,j} = double([0.5 0.5 0.5 0.5]);
        end
        
        %here we flip the vertical axes to the right position!
        shiftPicture((i-1)*length(x2)+1:i*length(x2),(j-1)*length(x1)+1:j*length(x2)) = fliplr((densityInControl-densityInWell)')';        
       
        donePercent = ((i-1)*colNum+j)/(rowNum*colNum);
        waitText = sprintf('Smooth well predictions...  %d%% done', int16(donePercent * 100));
        if ishandle(waitBarHandle)
            waitbar(donePercent, waitBarHandle, waitText);
        end
    end
end


if ishandle(waitBarHandle)
    close(waitBarHandle);
end

end

