function createRegressionPDFReport(results,plateName,targetFolder,resolution,controlRow,controlCol)
% AUTHOR:    Abel Szkalisity
% DATE:     July 11, 2016
% NAME:     createRegressionPDFReport
%
% To create a pdf report of the regression results.
%
% INPUT:
%   results         A cellarray containing the regression results. (result
%                   of prediction) 
%   plateName       Name of the plate, used in file name giving.
%   targetFolder    The folder where the pdf is saved
%   resolution      The regression plane's heatmap resolution.
%   controlRow      The index of the row of the well
%   controlCol      The index of the column of the well
%
%
% COPYRIGHT
% Advanced Cell Classifier (ACC) Toolbox. All rights reserved.
% Copyright (C) 2016 Peter Horvath,
% Synthetic and System Biology Unit, Hungarian Academy of Sciences,
% Biological Research Center, Szeged, Hungary; Institute for Molecular
% Medicine Finland, University of Helsinki, Helsinki, Finland.

figureForPDF = figure(1);

%plot for the regression heatmaps
regPlateAxes = subplot(3,1,1);

plateWithRegression = zeros([size(results).*(resolution+2),3]);

[r, c] = size(results);
yLabels = char('A'):char('A'+r-1); yLabels = yLabels';

for i=1:size(results,1)
    for j=1:size(results,2)
        if ~isempty(results{i, j})           
            wellIcon = plotCurrentRegression2D(results{i,j},resolution);
        else
            wellIcon = zeros(resolution+2, resolution+2, 3);
            wellIcon(:,:,:) = .25;
            wellIcon(2:end-1,2:end-1,:) = 0;
        end        
        plateWithRegression((i-1)*(resolution+2)+1:i*(resolution+2),(j-1)*(resolution+2)+1:j*(resolution+2),:) = wellIcon;
    end
end

imagesc(plateWithRegression);
colormap(regPlateAxes,hot(32));
colorbar();
title('Regression-heatmaps','FontSize',8);
set(gca,'YTick',(resolution+2)/2:(resolution+2):r*(resolution+2));
set(gca,'XTick',(resolution+2)/2:(resolution+2):c*(resolution+2));
set(gca,'XTickLabel',1:c)
set(gca,'YTickLabel',yLabels);
set(gca,'FontSize');
xlabel('col');
ylabel('row');
caxis([0 1]);

%do the plot for the arrows
subplot(3,1,2);
[rowNum,colNum] = size(results);
controlRowFlipped = rowNum - controlRow+1;

%here calculate the change vectors
[shiftVectors,shiftPicture] = calculateShiftVectors(results,controlRow,controlCol);
s = cell2mat(shiftVectors);
controlColumnFirst = s(:,1:4:end);
controlColumnSecond = s(:,2:4:end);
wellColumnFirst = s(:,3:4:end);
wellColumnSecond = s(:,4:4:end);

[first,second] = meshgrid(0:colNum-1,rowNum-1:-1:0);

quiver(first(:)+controlColumnFirst(:),second(:)+controlColumnSecond(:),wellColumnFirst(:)-controlColumnFirst(:),wellColumnSecond(:)-controlColumnSecond(:),'LineWidth',1.0,'AutoScale','off');

hold on;
plot([controlCol-1 controlCol controlCol controlCol-1 controlCol-1],[controlRowFlipped-1,controlRowFlipped-1,controlRowFlipped,controlRowFlipped,controlRowFlipped-1],'LineWidth',1.5,'Color','red');
axis([0 colNum 0 rowNum]);
plot([1:colNum; 1:colNum],[zeros(1,colNum); repmat(rowNum,1,colNum)],'k');
plot([zeros(1,rowNum); repmat(colNum,1,rowNum)],[1:rowNum; 1:rowNum],'k');

title('Shift of population centrum','FontSize',8);
set(gca,'YTick',0.5:1:rowNum);
set(gca,'XTick',0.5:1:colNum);
set(gca,'XTickLabel',1:c)
set(gca,'YTickLabel',fliplr(yLabels')');

subplot(3,1,3);
imagesc(shiftPicture);
hold on;
colormap(hot(32));
[ySize,xSize] = size(shiftPicture);
ySize2 = ySize./rowNum;
xSize2 = xSize./colNum;
plot([(1:colNum).*xSize2; (1:colNum).*ySize2],[zeros(1,colNum); repmat(rowNum.*ySize2,1,colNum)],'k');
plot([zeros(1,rowNum); repmat(colNum.*xSize2,1,rowNum)],[(1:rowNum).*ySize2 ; (1:rowNum).*xSize2],'k');
title('Difference of surfaces','FontSize',8);
set(gca,'YTick',ySize/(rowNum*2):ySize/rowNum:ySize);
set(gca,'XTick',xSize/(colNum*2):xSize/colNum:xSize);
set(gca,'XTickLabel',1:c);
set(gca,'YTickLabel',yLabels);

timestamp = clock;

outputName = [targetFolder filesep plateName '_' num2str(timestamp(1)) num2str(timestamp(2),'%02d') num2str(timestamp(3),'%02d') num2str(timestamp(4),'%02d') num2str(timestamp(5),'%02d') '_regressionReport' ];

print(figureForPDF, '-dpdf', outputName);

close(figureForPDF);




end