function regressionReportForPlate(plateNumber,html,thumbnail,resolution,baseFolder,idx,controlRow,controlCol)
% AUTHOR:    Abel Szkalisity
% DATE:     July 10, 2016
% NAME:     regressionReportForPlate
%
% Te encapsulate the process of generating reports of regression. We
% suppose that the target folder is already there, and there is a trained
% classifier and regressor. Within the baseFolder
% this function makes a subfolder (named time_platename).
%
% INPUT:
%   plateNumber     The # of the plate of which we make the prediction
%   html            bool value whether we want to make html report too
%   thumbnail       bool value if we generate thumbnails in html report.
%   resolution      resolution of the regression plane. (to how many parts one side of the plane is divided when the distribution is calculated)
%   baseFolder      the folder name where the report will be saved.
%   idx             The index of the regression plane to use
%   controlRow      The index of the row of the well
%   controlCol      The index of the column of the well
%
% OUTPUT:
%   The files will be written to the projectFolder [(CommonHandles.DirName)
%   filesep basefolder].
%
% COPYRIGHT
% Advanced Cell Classifier (ACC) Toolbox. All rights reserved.
% Copyright (C) 2016 Peter Horvath,
% Synthetic and System Biology Unit, Hungarian Academy of Sciences,
% Biological Research Center, Szeged, Hungary; Institute for Molecular
% Medicine Finland, University of Helsinki, Helsinki, Finland.

global CommonHandles;

colmax = CommonHandles.PlateTypeInfo{CommonHandles.WellType}.col;
rowmaxnum = CommonHandles.PlateTypeInfo{CommonHandles.WellType}.row;
rowmax = char(rowmaxnum+'A'-1);

%for the waitbar
wellcount = colmax*rowmaxnum;
counter = 0;

timestamp = clock;

tgtFolder = [CommonHandles.DirName filesep baseFolder filesep num2str(timestamp(1)) num2str(timestamp(2),'%02d') num2str(timestamp(3),'%02d') num2str(timestamp(4),'%02d') num2str(timestamp(5),'%02d') '_' CommonHandles.PlatesNames{plateNumber}];
mkdir(tgtFolder);

results = cell(rowmaxnum,colmax);

%open csv file
%do CSVs.
singleCellFID = fopen([tgtFolder filesep CommonHandles.PlatesNames{plateNumber} '_singleCellRegressionPositions.csv'],'w');

%for structured data
if colmax > -1;
    
    waitBarHandle = waitbar(0,'Prediction...');
    
    %header for csv    
    fprintf(singleCellFID, 'PlateName, Row, Col, ImageName, ImageNumber, ObjectNumber, xPixelPos, yPixelPos, regPosX, regPosY\n');
    
    %iterate through every well
    for col=1:colmax
        for row ='A':rowmax
            counter = counter+1;
            rownum = row - 'A' + 1;
            % create base file names
            colString = sprintf('%02d', col);
            FileNamePrefix =  [CommonHandles.DirName filesep char(CommonHandles.PlatesNames(plateNumber)) filesep  CommonHandles.MetaDataFolder filesep char(CommonHandles.PlatesNames(plateNumber)) '*_w' row colString '*'];
            MetadataFiles = dir(FileNamePrefix);
            
            predForWell = cell(1,size(MetadataFiles,1));
            
            for i=1:size(MetadataFiles,1)
                [~, fileNameexEx, ~] = fileparts(MetadataFiles(i).name);
                %read in features
                [~,featuresFull] = readMetaData(CommonHandles, fileNameexEx, plateNumber);
                features = featuresFull(:,3:end);
                %do classification
                [ out, ~, ~ ] = predictClassifier( features , CommonHandles);
                %select cells into regression
                features = features(out==idx,:);
                %check for regression cells' existnace
                if ~isempty(features)
                    predForWell{i} = CommonHandles.RegressionPlane{idx}.Predictor.predict(features');
                end
                
                regressionCellsIdx = find(out==idx);
                for cellIdx = 1:size(predForWell{i},2)
                    %Printing out to file PlateName, Row, Col, ImageName, ImageNumber, ObjectNumber, xPixelPos, yPixelPos, Class
                    fprintf(singleCellFID, '%s, %s, %d, %s, %d, %d, %f, %f, %f, %f\n', ...
                        char(CommonHandles.PlatesNames(plateNumber)), ...
                        row, ...
                        col, ...
                        fileNameexEx,...
                        i, ...
                        regressionCellsIdx(cellIdx), ...
                        featuresFull(regressionCellsIdx(cellIdx), 1),...
                        featuresFull(regressionCellsIdx(cellIdx), 2),...
                        predForWell{i}(1,cellIdx),...
                        predForWell{i}(2,cellIdx)...
                    );
                end
                
            end
            
            results{rownum,col} = cell2mat(predForWell);
            
            donePercent = counter./wellcount;
            waitText = sprintf('Perform regression...  %d%% done', int16(donePercent * 100));
            if ishandle(waitBarHandle)
                waitbar(donePercent, waitBarHandle, waitText);
            end
        end
    end
    
    %close regression waitbar
    if ishandle(waitBarHandle)
        close(waitBarHandle);
    end
    
    if html
        createRegressionHTML(results', CommonHandles.PlatesNames{plateNumber}, tgtFolder ,resolution,thumbnail);
    end
    
    createRegressionPDFReport(results, CommonHandles.PlatesNames{plateNumber},tgtFolder, resolution,controlRow,controlCol);
    
    %save resultn as mat file
    save([tgtFolder filesep 'regressionsByPlate.mat'],'results');
        
    
%unstructured data
else
end

%close csv file
fclose(singleCellFID);
    